/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include "dbitemcolumns.h"
#include "dbbase.h"



void DbItemColumns::SetColumns(PGresult * r)
{
	// PQfnumber returns -1 if there is no such a column
	id                = PQfnumber(r, "id");
	user_id           = PQfnumber(r, "user_id");
	group_id          = PQfnumber(r, "group_id");
	privileges        = PQfnumber(r, "privileges");
	date_creation     = PQfnumber(r, "date_creation");
	date_modification = PQfnumber(r, "date_modification");
	url               = PQfnumber(r, "url");
	type              = PQfnumber(r, "type");
	parent_id         = PQfnumber(r, "parent_id");
	content_id        = PQfnumber(r, "content_id");
	default_item      = PQfnumber(r, "default_item");
	subject           = PQfnumber(r, "subject");
	content           = PQfnumber(r, "content");
	content_type      = PQfnumber(r, "content_type");
	guest_name        = PQfnumber(r, "guest_name");
	html_template     = PQfnumber(r, "template");
	modification_user_id = PQfnumber(r, "modification_user_id");
	file_path         = PQfnumber(r, "file_path");
	file_fs           = PQfnumber(r, "file_fs");
	file_type         = PQfnumber(r, "file_type");
	has_thumb         = PQfnumber(r, "has_thumb");
}



void DbItemColumns::SetItem(PGresult * r, long row, Item & item)
{
	if( id                != -1 ) item.id                = DbBase::AssertValueLong(r, row, id);
	if( user_id           != -1 ) item.user_id           = DbBase::AssertValueLong(r, row, user_id);
	if( group_id          != -1 ) item.group_id          = DbBase::AssertValueLong(r, row, group_id);
	if( privileges        != -1 ) item.privileges        = DbBase::AssertValueInt(r, row, privileges);
	if( date_creation     != -1 ) item.date_creation     = DbBase::AssertValueTm(r, row, date_creation);
	if( date_modification != -1 ) item.date_modification = DbBase::AssertValueTm(r, row, date_modification);
	if( url               != -1 ) item.url               = DbBase::AssertValueWide(r, row, url);
	if( type              != -1 ) item.type              = static_cast<Item::Type>( DbBase::AssertValueInt(r, row, type) );
	if( parent_id         != -1 ) item.parent_id         = DbBase::AssertValueLong(r, row, parent_id);
	if( content_id        != -1 ) item.content_id        = DbBase::AssertValueLong(r, row, content_id);
	if( default_item      != -1 ) item.default_item      = DbBase::AssertValueLong(r, row, default_item);
	if( subject           != -1 ) item.subject           = DbBase::AssertValueWide(r, row, subject);
	if( content           != -1 ) item.content           = DbBase::AssertValueWide(r, row, content);
	if( content_type      != -1 ) item.content_type      = static_cast<Item::ContentType>( DbBase::AssertValueInt(r, row, content_type) );
	if( guest_name        != -1 ) item.guest_name        = DbBase::AssertValueWide(r, row, guest_name);
	if( html_template     != -1 ) item.html_template     = DbBase::AssertValueWide(r, row, html_template);
	if( modification_user_id != -1 ) item.modification_user_id = DbBase::AssertValueLong(r, row, modification_user_id);
	if( file_path         != -1 ) item.file_path         = DbBase::AssertValueWide(r, row, file_path);
	if( file_fs           != -1 ) item.file_fs           = DbBase::AssertValueInt(r, row, file_fs);
	if( file_type         != -1 ) item.file_type         = DbBase::AssertValueInt(r, row, file_type);
	if( has_thumb         != -1 ) item.has_thumb         = DbBase::AssertValueBool(r, row, has_thumb);
}




